@@ -23,3 +23,15 @@ setMethod(
     .Object
   }
 )
+
+setMethod(
+  "calculate_initializing_values", signature(object = "equation_directional"),
+  function(object) {
+    reg <- stats::lm(
+      object@dependent_vector ~ object@independent_matrix - 1,
+      subset = object@separation_subset
+    )
+    names(reg$coefficients) <- colnames(object@independent_matrix)
+    reg
+  }
+)
